%

% Copyright (c) 2014-2015,2018 LAAS/CNRS
% All rights reserved.
%
% Redistribution  and  use  in  source  and binary  forms,  with  or  without
% modification, are permitted provided that the following conditions are met:
%
%   1. Redistributions of  source  code must retain the  above copyright
%      notice and this list of conditions.
%   2. Redistributions in binary form must reproduce the above copyright
%      notice and  this list of  conditions in the  documentation and/or
%      other materials provided with the distribution.
%
% THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
% WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
% MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
% ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
% WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
% IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
%
%                                           Anthony Mallet on Thu Jun 11 2015
%
classdef request < handle
  properties (Access = private)
    component
    service
    id
    client
  end

  properties (SetAccess = ?genomix.client)
    status	% current execution status: 'sent', 'done' or 'error'.
    result	% regular request result when status is 'done'.
    exception	% exceptional result when status is 'error'.
  end

  methods (Access = ?genomix.client)
    function self = request(client, component, service, id)
      % genomix.request - Manage asynchronous requests (replies, status, ...)
      %
      % genomix.request Properties:
      %    status - current execution status: 'sent', 'done' or 'error'.
      %    result - regular request result when status is 'done'.
      %    exception - exceptional result when status is 'error'.
      %
      % genomix.request Methods:
      %    wait - Suspend program execution until a certain event is received.
      %    abort - Cancel an ongoing request.
      %
      self.component = component;
      self.service = service;
      self.id = id;
      self.client = client;

      self.status = 'none';
      self.result = [];
      self.exception = [];
    end
  end

  methods
    function delete(self)
      % delete - Abort an ongoing request.
      %
      try
        genomix_server('get', self.client.id, ...
                sprintf('/module/%s/clean/%s', self.component, self.id));
      catch
      end
    end

    function status = get.status(self)
      genomix_server('doevents', 0);
      status = self.status;
    end

    function result = get.result(self)
      switch self.status
        case {'none', 'sent'}
          genomix_server('doevents', 0);
      end
      result = self.result;
    end

    function exception = get.exception(self)
      switch self.status
        case {'none', 'sent'}
          genomix_server('doevents', 0);
      end
      exception = self.exception;
    end


    % --- wait -------------------------------------------------------------
    %
    function wait(self, timeout, expect)
      % wait - Suspend program execution until a certain event is received.
      %
      % obj.wait()
      %   Suspend program execution until the request obj is in the 'done'
      %   or 'error' state.
      % obj.wait(timeout)
      %   Same as above, with a timeout in seconds.
      % obj.wait(timeout, expect)
      %   Wait for the 'expect' event to occur. 'expect' must be one of
      %   'sent', 'done' or 'error'.
      %
      switch nargin
        case 1
          timeout = -1;
          expect = 'done';
        case 2
          expect = 'done';
      end

      maxtime = etime(clock, [2015 0 0 0 0 0]) + timeout;
      while true
        status = self.status;
        if strcmp(status, 'error') return; end
        if strcmp(status, expect) return; end
        if strcmp(expect, 'sent') && strcmp(status, 'done') return; end

        if timeout > 0
          tout = maxtime - etime(clock, [2015 0 0 0 0 0]);
          if tout < 0 error('timeout'); end
        elseif timeout == 0
          break;
        else
          tout = -1;
        end
        genomix_server('doevents', tout);
      end
    end

    % --- abort ------------------------------------------------------------
    %
    function abort(self)
      % abort - Cancel an ongoing request.
      %
      % obj.abort()
      %   Request an interrupt for the request 'obj'.
      %
      if isvalid(self.client)
        genomix_server('get', self.client.id, ...
                       sprintf('/module/%s/abort/%s', self.component, self.id));
      else
        e.message = 'client was deleted';
        e.stack.file = '';
        e.stack.line = 0;
        e.stack.name = strcat(self.component, '::abort(', self.service, ')');
        error(e);
      end
    end
  end
end
